# cache2k 1.9.2.Alpha "North Atlantic"

This is a preview release for evaluation purposes and should not be used in production.

The preview is in preparation for version 2 which will have some major breaking
changes to the previous releases. The final release will be **2.0.0.Final - "North Atlantic"**.

## New and Noteworthy

- @Nullable annotations
- General scheme to enable extra functionality, used for JMX and Micrometer support
- No caching of exceptions by default. This has to be enabled explicitly by specifying 
  `ResiliencePolicy`. Previous default resilience available as `UniversalResiliencePolicy` in 
  `cache2k-addon`

## Possible Breakages

Since this is a major release, some rarely used things in the API are 
changed without backwards compatibility. Most code is supposed to run
without breaking.

- `Cache.getStatistics` removed. Replaced by `CacheManagement.of(Cache).sampleStatistics()`.
- `Cache.clearAndClose` removed. Replaced by `CacheManagement.destroy`
- Removed deprecated `MutableCacheEntry.getCurrentTime`, replaced with `getStartTime`
- Remove `MutableCacheEntry.wasExisting` and `MutableCacheEntry.getOldValue` 
- Rename `MutableCacheEntry.getRefreshedTime` to `getModificationTime`
- `MutableCacheEntry.exists`: Does not change the value after `setValue` or triggered load
- `MutableCacheEntry.getValue`: Does not change the value after `setValue`
- Every class with `Configuration` shortened to `Config` and consequently changed the
  package name `org.cache2k.configuration` to `org.cache2k.config`.
- `Cache2kBuilder.toConfiguration()` renamed to `config()`
- Lots restructuring around the extra configuration sections and customizations
- remove `Cache2kBuilder.enableJmx`. Replaced with `Cache2kBuilder.enable(JmxSupport.class)`
- `Cache.requestInterface` throws `UnsupportedOperationException` if the requested interface
  is not supported instead of null
- Change in special expiry values semantics: Remove `ExpiryTimeValues.REFRESH`. 
  `ExpiryTimeValues.NOW` expires and  starts a refresh if refresh ahead is enabled. 
  `ExpiryTimeValues.NO_CACHE` expires and starts no refresh.
- No caching of exceptions by default. This has to be enabled explicitly by specifying 
  `ResiliencePolicy`. Previous default resilience available as `UniversalResiliencePolicy` in 
  `cache2k-addon`

 end::tagname[] 
 
## API Changes 

- Renamed `ExceptionInformation` to `LoadExceptionInfo`
- New method `CacheEntry.getExceptionInfo()`
- `org.cache2k.jmx`, API for JMX MXBeans removed
- New interfaces `CacheManagement`, `CacheControl` and `CacheInfo` in `org.cache2k.management`
- New method: `MutableCacheEntry.lock`
- Rename `MutableCacheEntry.reload` to `MutableCacheEntry.load`
- Remove `TimeReference` interface from public API and make that local to `cache2k-core`
- New method: `MutableCacheEntry.getExpiryTime()`
- Remove `Cache2kBuilder.enableJmx`
- `Cache2kBuilder`: New methods `set
- New methods: `Cache.mutate`, `Cache.mutateAll`

## Fixes and Improvements

- `Cache.loadAll()` and `Cache.reloadAll()` complete with exception if a a loader
  exception happened. If more than one exception happened the one is propagated.
- Improved interface of `ExceptionPropagator` and `LoadExceptionInfo`,
  A `CacheEntry` may be cast to `LoadExceptionInfo` if an exception is present
- `Cache.invoke`, `EntryProcessor`: Make expiry calculations and listeners calls more consistent
  if expiry event races with the start of the operation
- `MutableCacheEntry.lock` allows to lock an entry for mutation explicitly
- `Cache.asMap` / `ConcurrentMap` implementation implements compute methods
  and runs them only once 
- Introduced `ConfigAugmenter` which can make changed to the configuration before a cache is build 
- `Cache2kBuilder`: Methods `setup`, `apply` allow to execute a function on the configuration
  or builder, which can be used to factor out configuration fragments.
  We use this to provide `enable` and `disable` methods on policies.
- AdvancedCacheLoader, AsyncCacheLoader and ExpiryPolicy get null if current entry carries exception, https://github.com/cache2k/cache2k/issues/163
- Remove ServiceLoader from JMX support in `cache2k-jmx`, `cache2k-micrometer`
- Introduce concept of `CacheFeature` and `ToggleCacheFeature`
- `CacheType`: immutable instead of bean. `CacheType.of` moved from `CacheTypeCapture`
- Remove the need of serializable from the configuration objects, by doing a deep
  copy of the default configuration via bean setters and getters.
- `Cache2kConfig`: Make `setEternal` a separate flag. 
   Remove logic from setters, so we can use setters and getter to copy the
   object
- `Cache.mutate`, `Cache.mutateAll` for a mutation only entry processor variant
- `@Nullable`, `@NonNull` annotations
- Within alpha: change new ExceptionPropagator from `<K>` to `<K, V>`, dito at `LoadExceptionInfo`
- @Nullness annotations
- Lots of typing refinements
- `Cache.entries` and `Cache.keys` return a `Set`
- `Cache.computeIfAbsent` with function
- Ignore `entryCapacity` setting in case a weigher is specified
- Allow listeners to throw a checked exception
